<?php
/**
 * @author Valentin Alisch <hallo@valentinalisch.de>
 * @version 2.5.5
 */



/**
 * Setup and fail safe in all situations
 *
 * @since 1.0
 */
class Empty_Setup {
	/**
	 * Check and set options
	 *
	 * @since 1.0.1		Set current breakpoint
	 * @since 1.0
	 *
	 * @return void
	 */
	public function __construct() {
		$this->check_breakpoints();
		$this->set_current_breakpoint();
	}
	
	
	
	/**
	 * Check breakpoints
	 *
	 * @since 1.0
	 *
	 * @return boolean
	 */
	private function check_breakpoints() {
		$breakpoints = get_option( 'empty_breakpoints' );
		if ( ! empty( $breakpoints ) ) return true;
		
		$breakpoints = array(
			array(
				'slug'		=> 'sm',
				'title'		=> 'Small',
				'break'		=> array(
					'value'		=> 0,
					'unit'		=> 'px'
				)
			),
			array(
				'slug'		=> 'md',
				'title'		=> 'Medium',
				'break'		=> array(
					'value'		=> 768,
					'unit'		=> 'px'
				)
			),
			array(
				'slug'		=> 'lg',
				'title'		=> 'Large',
				'break'		=> array(
					'value'		=> 1024,
					'unit'		=> 'px'
				)
			)
		);
		
		return update_option( 'empty_breakpoints', $breakpoints );
	}
	
	/**
	 * Set current breakpoint
	 *
	 * @since 1.0.1
	 *
	 * @return boolean
	 */
	private function set_current_breakpoint() {
		$current_breakpoint = get_option( 'empty_breakpoints_current' );
		$helper = new Empty_Breakpoints();
		if ( ! empty( $current_breakpoint ) && $helper->get_index( $current_breakpoint ) !== false ) return true;
		
		$first = $helper->get_first();
		return update_option( 'empty_breakpoints_current', $first->slug );
	}
}

?>